"use strict";

exports.__esModule = true;
exports.default = void 0;

var DriverDOM = _interopRequireWildcard(require("driver-dom"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var STYLE = 'style';
var DEFAULT_VIEWPORT = 750;
var NON_DIMENSIONAL_REG = /opa|ntw|ne[ch]|ex(?:s|g|n|p|$)|^ord|zoo|grid|orp|ows|mnc|^columns$|bs|erim|onit/i;
var driver = Object.assign({}, DriverDOM, {
  createElement: function createElement(type, props, component) {
    if (props.hasOwnProperty(STYLE)) {
      var _extends2;

      var style = props[STYLE];
      var node = DriverDOM.createElement(type, _extends({}, props, (_extends2 = {}, _extends2[STYLE] = null, _extends2)), component);
      driver.setStyle(node, style);
      return node;
    } else {
      return DriverDOM.createElement(type, props, component);
    }
  },
  addEventListener: function addEventListener(node, eventName, eventHandler, props) {
    eventName = normalizeEventName(node, eventName, props);
    return DriverDOM.addEventListener(node, eventName, eventHandler, props);
  },
  removeEventListener: function removeEventListener(node, eventName, eventHandler, props) {
    eventName = normalizeEventName(node, eventName, props);
    return DriverDOM.removeEventListener(node, eventName, eventHandler, props);
  },
  setStyle: function setStyle(node, style) {
    if (Array.isArray(style)) {
      style = style.reduce(function (prev, curr) {
        return Object.assign(prev, curr);
      }, {});
    }

    var tranformedStyle = transformStyle(style); // Fist use DriverDOM set standard style.

    DriverDOM.setStyle(node, tranformedStyle); // Second process flex compatible style, like {display: ["-webkit-box", "-webkit-flex", "flex"]}.

    for (var prop in tranformedStyle) {
      if (tranformedStyle.hasOwnProperty(prop)) {
        var transformValue = tranformedStyle[prop]; // Hack handle compatibility issue

        if (Array.isArray(transformValue)) {
          for (var i = 0; i < transformValue.length; i++) {
            node.style[prop] = transformValue[i];
          }
        }
      }
    }
  }
});

function transformStyle(style, ret) {
  if (ret === void 0) {
    ret = {};
  }

  // Driver universal only process flex box compatible.
  // Process convert unit in DriverDOM
  for (var prop in style) {
    if (style.hasOwnProperty(prop)) {
      var val = style[prop];

      if (typeof val === 'object') {
        delete style[prop];
        transformStyle(val, ret);
      } else if (typeof val === 'number' && !NON_DIMENSIONAL_REG.test(prop)) {
        // append `rpx` to relevant styles
        ret[prop] = val + 'rpx';
      } else {
        ret[prop] = val;
      }
    }
  }

  return ret;
}

function normalizeEventName(node, eventName, props) {
  var tagName = node.tagName.toLowerCase();

  if (eventName === 'change' && (tagName === 'textarea' || tagName === 'input' && (!props.type || props.type === 'text' || props.type === 'password'))) {
    eventName = 'input';
  } else if (eventName === 'doubleclick') {
    eventName = 'dblclick';
  }

  return eventName;
}

var _default = driver;
exports.default = _default;